/**
 * Simple application which set up UART modules
 * wait for two bytes and send back the bigger one.
 */
module uart(
    input           CLOCK_50,
    input   [3:0]   KEY,
    input   [9:0]   SW,
    inout   [1:0]   GPIO_0,
    output  [9:0]   LEDR
);
    
    wire TX;
    wire RX;

    /* Assign UART pins. (RX = left, TX = right) */    
    assign GPIO_0[0]    = TX;    
    assign RX           = GPIO_0[1];
    
    /* Create UART clocks with max baudrate (115.2 kHz). */
    wire uart_transmitter_clock;
    wire uart_receiver_clock;
    
    clock_divider _uart_transmitter_clock(
        .clock(CLOCK_50),
        .out(uart_transmitter_clock)
    );
    
    clock_divider _uart_receiver_clock(
        .clock(CLOCK_50),
        .out(uart_receiver_clock)
    );
    
    defparam _uart_transmitter_clock.divider = 434;
    defparam _uart_receiver_clock.divider = 62;

    /* Predefine UART registers and wires. */
    reg[7:0]    uart_transmit_data;
    reg         uart_transmit_start;
    wire        uart_transmit_ready;
    
    reg         uart_receiver_clear;
    wire[7:0]   uart_receiver_data;
    wire        uart_receiver_ready;
    
    /* Set UART modules. */
    transmiter _trensmiter(
        .clock(uart_transmitter_clock),
        .start(uart_transmit_start),
        .data(uart_transmit_data),
        .out(TX),
        .ready(uart_transmit_ready)
    );
    
    receiver _receiver(
        .in(RX),
        .clock(uart_receiver_clock),
        .clear_flag(uart_receiver_clear),
        .out(uart_receiver_data),
        .ready(uart_receiver_ready)
    );
    
    /* Define state machine. */
    reg[2:0]    state;
    localparam  state_wait_for_first    = 32'd0;
    localparam  state_first_received    = 32'd1; 
    localparam  state_wait_for_second   = 32'd2;
    localparam  state_second_received   = 32'd3;
    localparam  state_send_data         = 32'd4;
    localparam  state_data_sent         = 32'd5;
    
    /* Helping registers. */
    reg[7:0]    first_byte;
    reg[7:0]    second_byte;
    
    /* Debug. */
    reg[7:0]    LED;
    
    assign LEDR = LED;
    
    /* So USB can read data. */
    reg[31:0] delay;
    
    localparam second_delay = 115200; 
    
    /* Main loop */ 
    always @(posedge uart_transmitter_clock)
    begin        
        LED <= state;
        
        case (state)
            state_wait_for_first:
            begin
                if (uart_receiver_ready == 1)
                begin 
                    state                   <= state_first_received;
                    first_byte              <= uart_receiver_data;
                    uart_receiver_clear     <= 1;
                end
            end
            state_first_received:
            begin
                state                   <= state_wait_for_second;
                uart_receiver_clear     <= 0;
            end
            state_wait_for_second:
            begin
                if (uart_receiver_ready == 1)
                begin 
                    state                   <= state_second_received;
                    second_byte             <= uart_receiver_data;
                    uart_receiver_clear     <= 1;
                end
            end
            state_second_received:
            begin
                state                   <= state_send_data;
                uart_receiver_clear     <= 0;
                delay                   <= second_delay;
            end
            state_send_data:
            begin
                if (delay > 0)
                begin
                    delay <= delay - 1;
                end
                
                if (delay == 0 && uart_transmit_ready == 1)
                begin
                    state               <= state_data_sent; 
                    uart_transmit_data  <= (first_byte > second_byte) ? first_byte : second_byte;
                    uart_transmit_start <= 1;
                end
            end
            state_data_sent:
            begin
                if (uart_transmit_ready == 0)
                begin
                    state               <= state_wait_for_first;
                    uart_transmit_start <= 0;
                end
            end
        endcase
    end

endmodule
